// rob@faludi.com
// (sound meter code adapted from Amit Pitaru's Sonia examples)

import pitaru.sonia_v2_9.*;   // Sonia 2.9 requires Processing 90
import mkv.MyGUI.*;           // MyGUI loses features before Processing 91
import java.io.*;
import java.net.*;

Sample rawSample; // The Sonia Sample object which we'll record into
Sample[] trimmedSample;  // a sample that's just the length of what was recorded
float startTime, stopTime;  // for measuring the length of a recording
String netID, keyboardInput, message; // text input and output
int recordMode, recTimeSec;   // numbers each screen
int counter; // general purpose counter
boolean recording, playing, saving; // holds current status
char[] modeList = {
  'C', 'N', 'F', 'L', 'P', 'D'};  // so I could use letters instead of numbers to signify which screen is current
String[] filename = {
  "firstname", "lastname", "placename"};
color backgroundColor, textColor, dataBoxColor, meterColor;
boolean readyToRecord, showButtons, filesSent;
PFont theFont, titleFont;

//////////  BUTTON INITIALIZE ///////////
MyGUI guiRed, guiGreen, guiNeutral;             // easier for each color of button to have its own GUI file
MyGUIStyle styleRed, styleGreen, styleNeutral;  // easier for each color of button to have its own GUI file
MyGUIButton recButton, playButton, backButton, nextButton, clearButton; // all of the buttons
//MyGUILabel label;                             // can't use labels with Processing 90  (but need it for Sonia)
//MyGUITextInput input;                         // can't use text inputs with Processing 90
// these integers are all required for MyGUIObjects
int fontSize, theBackground, buttonFace, buttonHighlight, buttonShadow, buttonText;
int icon, face, highlight, shadow, disabled, highlightText, scrollBar;
int buttonLocY, buttonSize;  // postion and dimension attributes of the buttons

////////// NETWORK INITIALIZE ///////////
int port = 8189;
String host = "sugaki.faludi.com";

void setup(){ 
  size(600,400);
  smooth();
  Sonia.start(this); 
  LiveInput.start();  // start the liveInput engine (see liveInput example)
  recTimeSec = 30; // number of seconds to record, determines the sample's max size. 
  rawSample = new Sample(44100*recTimeSec); // Create an empty Sample object with 44100*n-seconds frames (n seconds of data). 
  trimmedSample = new Sample[modeList.length];
  netID = "";
  keyboardInput = "";
  message = "";
  recordMode = 0;
  backgroundColor = color(255,255,255);
  textColor = color(0,0,0);
  dataBoxColor = color(200,200,200);
  meterColor = backgroundColor;
  theFont = loadFont("Futura-Medium-48.vlw"); 
  titleFont = loadFont("BertramLET-48.vlw");
  textFont(theFont);

  //////////  BUTTON SETUP ///////////
  fontSize = 14;
  buttonLocY = 300;  // location for all the buttons
  buttonSize = 60;
  theBackground = color(150,0,150);      // ??? this is the button background, but I've never seen it displayed
  buttonFace =  color(225,0,0);       // color of the button
  buttonHighlight = color(225,0,0);   // ring around the button that stays after clicking on it
  buttonShadow = color(20,20,20);       // a border around the outside of the button
  buttonText = color(0,0,0);          // color of the text on the button
  icon = color(0,0,0);
  face = color(150,150,150);
  highlight = color(225,0,0);        // color when button is depressed (or hovered after pressing for some reason)
  shadow = color(150,150,150);
  disabled = color(30,30,30);        // color when button is disabled
  highlightText = color(150,150,150);
  scrollBar = color(150,150,150);

  styleRed = new MyGUIStyle(this, theFont,  fontSize,  theBackground,  buttonFace,  buttonHighlight,  buttonShadow,
  buttonText, icon, face,  highlight,  shadow,  disabled,  highlightText,  scrollBar);

  buttonFace =  color(0,225,0);
  buttonHighlight = color(0,225,0);
  highlight = color(0,225,0);
  styleGreen = new MyGUIStyle(this, theFont,  fontSize,  theBackground,  buttonFace,  buttonHighlight,  buttonShadow,
  buttonText, icon, face,  highlight,  shadow,  disabled,  highlightText,  scrollBar);

  buttonFace =  color(225,225,225);
  buttonHighlight = color(225,225,225);
  highlight = color(225,225,225);
  styleNeutral = new MyGUIStyle(this, theFont,  fontSize,  theBackground,  buttonFace,  buttonHighlight,  buttonShadow,
  buttonText, icon, face,  highlight,  shadow,  disabled,  highlightText,  scrollBar);

  guiRed = new MyGUI(this, styleRed);
  guiGreen = new MyGUI(this, styleGreen);
  guiNeutral = new MyGUI(this, styleNeutral);
  recButton = new MyGUIButton(this,width/5*3,buttonLocY,"Record",buttonSize,buttonSize);
  playButton = new MyGUIButton(this,width/5*2,buttonLocY,"Play",buttonSize,buttonSize);
  backButton = new MyGUIButton(this,width/5*1,buttonLocY,"Back",buttonSize,buttonSize);
  nextButton = new MyGUIButton(this,width/5*4,buttonLocY,"Next",buttonSize,buttonSize);
  clearButton = new MyGUIButton(this,width/5*1,buttonLocY,"Clear",buttonSize,buttonSize);
  recButton.setActionCommand("Record");
  playButton.setActionCommand("Play");
  backButton.setActionCommand("Back");
  nextButton.setActionCommand("Next");
  clearButton.setActionCommand("Clear");
  playButton._disabled = true;
  backButton._disabled = false;
  nextButton._disabled = true;
  guiRed.add(recButton);
  guiGreen.add(playButton);
  guiNeutral.add(backButton);
  guiNeutral.add(nextButton);
  guiNeutral.add(clearButton);

} 


void draw(){
  background(backgroundColor);
  fill(25,25,150);
  stroke(textColor);
    textFont(titleFont);
  textSize(60);
  textAlign(CENTER);
  text("Voice Recorder",width/2,height/6);
    textFont(theFont);
  textSize(24);
  fill(textColor);
  text(message,width/2,height/20*9);
  getMeterLevel();
  recordModeSelect();
} 

void mousePressed() {
  // handle the first click that gives the applet focus, after which do nothing
  if (modeList[recordMode] == 'C') {
    recordMode++;
  }
}

void startRecording(){ 
  delay(200);                  // slight delay so that sound of mouse is not recorded
  meterColor = color(150,0,0);
  LiveInput.startRec(rawSample); // Record LiveInput data into the Sample object. 
  startTime = millis();          // note the time that recording started
} 

void stopRecording(){ 
  meterColor = backgroundColor;
  LiveInput.stopRec(rawSample);
  stopTime = millis();
  float recTime = (stopTime - startTime);
  int trimmedFrames = int(44100*(recTime/1000)-1000);
  trimmedSample[recordMode] = new Sample(trimmedFrames);
  float[] dataArray; 
  dataArray = new float[trimmedFrames];
  rawSample.read(dataArray, 0, 0, trimmedFrames);
  trimmedSample[recordMode].write(dataArray);
} 

void startPlay() {
  meterColor = color(0,150,0);
  trimmedSample[recordMode].play();
}

void stopPlay() {
  meterColor = backgroundColor;
  trimmedSample[recordMode].stop();
}


// ACCEPT KEYBOARD INPUT
void keyPressed() {
  if(modeList[recordMode] == 'N') {     // if we are currently accepting keyboard input
    if (key == ENTER && keyboardInput.length() >= 3) {
    netID = keyboardInput;
    recordMode++;
    }
    if (key != CODED && key != ENTER) {      // (the docs say that the enter key is coded, but it's not)
      if (key == BACKSPACE && keyboardInput.length() > 0) {       // handle deleting text
        keyboardInput = keyboardInput.substring(0,keyboardInput.length()-1); // only delete text if it exists
      }
      else {                     // add each keypress to the previous ones
        keyboardInput = keyboardInput+key;
        netID = "";  // clear the previous keyword
      }
    }
  }
}


void showButtons(boolean showButtons) {
  if(showButtons) {
    clearButton._visible=false;
    backButton._visible=true;
    playButton._visible=true;
    recButton._visible=true;
    nextButton._visible=true;
  }
  else {
    clearButton._visible=false;
    backButton._visible=false;
    playButton._visible=false;
    recButton._visible=false;
    nextButton._visible=false;
  }
}



void saveFiles() {
  for( int i=2; i<trimmedSample.length-1; i++) {
    trimmedSample[i].saveFile(netID+"-"+filename[i-2]);
    println("Saved: "+netID+"-"+filename[i-2]);
  }
}


public void stop(){ 
  Sonia.stop(); 
  super.stop(); 
} 
